import handler from './handler.js';
const endpoints = [
    'GET', 'POST', 'PUT', 'PATCH', 'DELETE', 'OPTIONS'
];
export function createZeroApi(config) {
    function createDirectory(directory) {
        let route = ''; // Set in Proxy
        /** api.users.user$(id). */
        function handleDirectory() {
            const isSlug = route.match(/\$$/g);
            if (isSlug)
                return function sluggedRoute(slug) {
                    return createDirectory({ path: directory.path + '/' + slug });
                };
            directory[route] = createDirectory({ path: directory.path + '/' + route });
            return directory[route];
        }
        /** api.users.post({ body: {...} }) */
        function handleEndpoint() {
            const method = route.toUpperCase();
            const path = directory.path;
            directory[route] = (contents, _fetch = null) => handler({ config, path, fetch: _fetch || fetch }, { method, headers: {}, ...contents });
            return directory[route];
        }
        return new Proxy(directory, {
            get(directory, _route) {
                route = _route;
                if (directory[route])
                    return directory[route];
                if (endpoints.includes(route))
                    return handleEndpoint();
                return handleDirectory();
            }
        });
    }
    return createDirectory({ path: '' });
}
